
//
//  CpuX86.cpp - (CPU X86) 
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 2 Jun 98 - James Leiterman
//
//  Last Modification
//
//      23 Jul 2001 - JCL - Brought up to date.
//		23 Apr 2008 - JCL - Brought up to date with VC 2005.
//
//  stab=4
//

#include <iostream>

using namespace std;

#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries

#include <string.h>
#include <assert.h>
#include <stdio.h>

    //  Book Development Library Modules

#include "CpuAsm.h"



//
//  CPU Vendor Lookup Table
//

static char *CpuVendor[] =
{
    "UNKNOWN",
    "INTEL",        // CPUVEN_INTEL     = 1
    "AMD",          // CPUVEN_AMD       = 2
    "CYRIX",        // CPUVEN_CYRIX     = 3
    "CENTAUR",      // CPUVEN_CENTAUR   = 4
    "NATIONAL",     // CPUVEN_NATIONAL  = 5
    "UMC",          // CPUVEN_UMC       = 6
    "NEXGEN",       // CPUVEN_NEXGEN    = 7
    "RISE",         // CPUVEN_RISE      = 8
    "TRANSMETA",    // CPUVEN_TRANSMETA = 9
};

extern "C" unsigned short SerialNum[6];
extern "C" void CpuDetectAsm( CpuInfo * const pInfo );



#ifdef CC_VMP_WIN32
vmp_SIMDEntryProc           vmp_SIMDEntry;
vmp_SIMDExitProc            vmp_SIMDExit;

void vmp_SIMDEntryGeneric(void){}
void vmp_SIMDExitGeneric(void){}
#endif



void CpuDetect( CpuInfo * const pcinfo )
{
    ASSERT_PTR4(pcinfo);
    
    CpuDetectAsm(pcinfo);

    if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        vmp_SIMDEntry =         vmp_SIMDEntryAsm3DNow;
        vmp_SIMDExit =          vmp_SIMDExitAsm3DNow;
      }

        // MMX, SSE, SSE2
    else if ( (CPUBITS_MMX | CPUBITS_SSE | CPUBITS_SSE2 ) & pcinfo->nBits )
      {
        vmp_SIMDEntry =         vmp_SIMDEntryAsmMMX;
        vmp_SIMDExit =          vmp_SIMDExitAsmMMX;
      }

    else
      {
        vmp_SIMDEntry =         vmp_SIMDEntryGeneric;
        vmp_SIMDExit =          vmp_SIMDExitGeneric;
      }
}



//
//  CPU information string
//

char *CpuInfoStr( char *pBuf, const CpuInfo * const pInfo )
{
    char *p;

    ASSERT_PTR(pBuf);
    ASSERT_PTR(pInfo);
    
    p = pBuf + sprintf( pBuf, "CpuId:%u", pInfo->nCpuId );
    p += sprintf( p, " '%s' ", CpuVendor[ pInfo->nMfg ] );

    if ( CPUBITS_FPU & pInfo->nBits )
    {
        p += sprintf( p, " FPU" );
    }

    if ( CPUBITS_MMX & pInfo->nBits )
    {
        p += sprintf( p, " MMX" );
    }

    if ( CPUBITS_FXSR & pInfo->nBits )
    {
        p += sprintf( p, " FXSR" );
    }

    if ( CPUBITS_3DNOW & pInfo->nBits )
    {
        p += sprintf( p, " 3DNow!" );
    }

    if ( CPUBITS_3DNOW_EXT & pInfo->nBits )
    {
        p += sprintf( p, " +Ext" );
    }

    if ( CPUBITS_3DNOW_MMX & pInfo->nBits )
    {
        p += sprintf( p, " +MMX_Ext" );
    }

    if ( CPUBITS_SSE & pInfo->nBits )
    {
        p += sprintf( p, " SSE" );
    }

    if ( CPUBITS_SSE2 & pInfo->nBits )
    {
        p += sprintf( p, " SSE2" );
    }

    if ( CPUBITS_SSE3 & pInfo->nBits )
    {
        p += sprintf( p, " SSE3" );
    }

    if ( CPUBITS_EM64T & pInfo->nBits )
    {
        p += sprintf( p, " EM64T" );
    }

    else if ( CPUBITS_AMD64 & pInfo->nBits )
    {
        p += sprintf( p, " AMD64" );
    }

    if ( CPUBITS_HTT & pInfo->nBits )
    {
        p += sprintf( p, " HTT" );
    }

	if ( 1 < pInfo->nProcCnt)			// Hyperthreaded?
	{
		p += sprintf( p, " Hyper#%u", pInfo->nProcCnt );
	}

    p += sprintf( p, "\n" );

    if ( CPUBITS_3DNOW_SSE & pInfo->nBits )
    {
        p += sprintf( p, "3DNow! Professional\n" );
    }

#if 01
    if (SerialNum[0])
      {
        p += sprintf(p, "SerialNo. ");
        sprintf( p, "%4.4x-%4.4x-%4.4x-%4.4x-%4.4x-%4.4x\n",
            SerialNum[5], SerialNum[4], SerialNum[3], SerialNum[2], SerialNum[1], SerialNum[0] );
        _strupr(p);
      }     
#endif

    return pBuf;
}

