
//
//  PBoolX86.cpp - Packed Boolean Logic
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 27 Dec, 2001 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream>

    //  Book Development Library Modules

#include "CpuAsm.h"    
#include "vmp3D.h"
#include "vmp3DEx.h"
#include "PBool.h"
#include "PBoolEx.h"



//
//  Setup Packed Boolean Logic
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void PBoolSetup( const CpuInfo * const pcinfo )
{
    ASSERT_PTR4(pcinfo);
        
    if ( CPUBITS_SSE2 & pcinfo->nBits )
      {
        vmp_pand =              vmp_pandAsmSSE2;
        vmp_pand0 =             vmp_pandAsmSSE20;
        vmp_pandc =             vmp_pandcAsmSSE2;
        vmp_pandc0 =            vmp_pandcAsmSSE20;
        vmp_por =               vmp_porAsmSSE2;
        vmp_por0 =              vmp_porAsmSSE20;
        vmp_pxor =              vmp_pxorAsmSSE2;
        vmp_pxor0 =             vmp_pxorAsmSSE20;
        vmp_pnor =              vmp_pnorAsmSSE2;
        vmp_pnor0 =             vmp_pnorAsmSSE20;
        vmp_pnot =              vmp_pnotAsmSSE2;
        vmp_pnot0 =             vmp_pnotAsmSSE20;
      }

    else if ( CPUBITS_MMX & pcinfo->nBits )
      {
        vmp_pand =              vmp_pandAsmMMX;
        vmp_pand0 =             vmp_pandAsmMMX;
        vmp_pandc =             vmp_pandcAsmMMX;
        vmp_pandc0 =            vmp_pandcAsmMMX;
        vmp_por =               vmp_porAsmMMX;
        vmp_por0 =              vmp_porAsmMMX;
        vmp_pxor =              vmp_pxorAsmMMX;
        vmp_pxor0 =             vmp_pxorAsmMMX;
        vmp_pnor =              vmp_pnorAsmMMX;
        vmp_pnor0 =             vmp_pnorAsmMMX;
        vmp_pnot =              vmp_pnotAsmMMX;
        vmp_pnot0 =             vmp_pnotAsmMMX;
      }

    else
      {
        vmp_pand =              vmp_pandAsm;
        vmp_pand0 =             vmp_pandAsm;
        vmp_pandc =             vmp_pandcAsm;
        vmp_pandc0 =            vmp_pandcAsm;
        vmp_por =               vmp_porAsm;
        vmp_por0 =              vmp_porAsm;
        vmp_pxor =              vmp_pxorAsm;
        vmp_pxor0 =             vmp_pxorAsm;
        vmp_pnor =              vmp_pnorAsm;
        vmp_pnor0 =             vmp_pnorAsm;
        vmp_pnot =              vmp_pnotAsm;
        vmp_pnot0 =             vmp_pnotAsm;
      }
}




