        TITLE   Book pboolX86.asm - Packed Boolean Logic (X86)
        PAGE    53,160

;
; pboolX86.asm - Packed Boolean Logic
;
; This document contains file that you may use at your own risk in your own programs.
;
; This module is designed to accomodate X86 processors in the book.
;
;       NOTE: Needs masm 6.14 or later!
;
;       Created - 19 Dec 2001 - J.Leiterman
;
;       Modifications
;
;               
;
;       Tabs = 8



        .XLIST

        .387
        .686P
        .MMX
        .model flat, C

        include x86.inc
        include KariType.inc
 
        .LIST



        .data

NotMaskQ dq     0ffffffffffffffffh,0ffffffffffffffffh   ;128 Flipping Bits
NotMaskO OWORD  0ffffffffffffffffffffffffffffffffh   ;128 Flipping Bits



pboolX86  SEGMENT USE32 PUBLIC 'CODE'



;_________________________________________________________________________________________________
;=================================================================================================
;       Logical NOT  D=(!A)
;
;void vmp_pnot(  void * const pvD, const void * const pvA )
;{
;       *(((uint32*)pvD)+0) = 0xffffffff ^ *(((uint32*)pvA)+0);
;       *(((uint32*)pvD)+1) = 0xffffffff ^ *(((uint32*)pvA)+1);
;       *(((uint32*)pvD)+2) = 0xffffffff ^ *(((uint32*)pvA)+2);
;       *(((uint32*)pvD)+3) = 0xffffffff ^ *(((uint32*)pvA)+3);
;}
;       *** X86 version *** 128bit (un)aligned

        .586
        align   16
        public  vmp_pnotAsm
vmp_pnotAsm       proc    near pvD:ptr, pvA:ptr
        push    ebx

ASSERT_PTR(pvA)
ASSERT_PTR(pvD)

        mov     eax,pvA
        mov     ebx,pvD

        mov     ecx,[eax+0]             ; Read lower 64bits
        mov     edx,[eax+4]                             
        not     ecx
        not     edx
        mov     [ebx+0],ecx             ; Write lower 64bits
        mov     [ebx+4],edx

        mov     ecx,[eax+8]             ; Read upper 64bits
        mov     edx,[eax+12]                            
        not     ecx
        not     edx
        mov     [ebx+8],ecx             ; Write upper 64bits
        mov     [ebx+12],edx

        pop     ebx
        ret
vmp_pnotAsm     endp



;_________________________________________________________________________________________________
;       *** MMX version *** 128bit (un)aligned

        align   16
        .MMX    
        public  vmp_pnotAsmMMX
vmp_pnotAsmMMX       proc    near pvD:ptr, pvA:ptr
;;;        emms
ASSERT_PTR(pvA)
ASSERT_PTR(pvD)

        mov     eax,pvA
        mov     edx,pvD

        movq    mm0,[eax+0]                     ; Read Data
        movq    mm1,[eax+8]                             

        pxor    mm0,NotMaskQ                    ; Flip Bits
        pxor    mm1,NotMaskQ

        movq    [edx+0],mm0                     ; Write Data
        movq    [edx+8],mm1

;;;        emms
        ret
vmp_pnotAsmMMX  endp



;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit non aligned

        align   16
        .XMM
        public  vmp_pnotAsmSSE20
vmp_pnotAsmSSE20       proc    near pvD:ptr, pvA:ptr

ASSERT_PTR(pvA)
ASSERT_PTR(pvD)

        mov     eax,pvA

        movdqu  xmm0,[eax]                      ; Read Data
        mov     eax,pvD

        pxor    xmm0,NotMaskO                   ; Flip Bits
        movdqu  [eax],xmm0                      ; Write Data

        ret
vmp_pnotAsmSSE20        endp


;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit aligned

        align   16
        .XMM
        public  vmp_pnotAsmSSE2
vmp_pnotAsmSSE2       proc    near pvD:ptr, pvA:ptr

ASSERT_PTR16(pvA)
ASSERT_PTR16(pvD)

        mov     eax,pvA

        movdqa  xmm0,[eax]                      ; Read Data
        mov     eax,pvD

        pxor    xmm0,NotMaskO                   ; Flip Bits
        movdqa  [eax],xmm0                      ; Write Data

        ret
vmp_pnotAsmSSE2 endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Logical AND  D=(A & B)
;
;void vmp_pand( void * const pvD, const void * const pvA, const void * const pvB )
;{
;       *(((uint32*)pvD)+0) = *(((uint32*)pvA)+0) & *(((uint32*)pvB)+0);
;       *(((uint32*)pvD)+1) = *(((uint32*)pvA)+1) & *(((uint32*)pvB)+1);
;       *(((uint32*)pvD)+2) = *(((uint32*)pvA)+2) & *(((uint32*)pvB)+2);
;       *(((uint32*)pvD)+3) = *(((uint32*)pvA)+3) & *(((uint32*)pvB)+3);
;}
;       *** X86 version *** 128bit (un)aligned

        align   16
        .586
        public  vmp_pandAsm
vmp_pandAsm       proc    near  ; pvD:ptr, pvA:ptr, pvB:ptr
        push    ebp
        push    ebx
        push    esi
        push    edi

        mov     eax,[esp+16+8]          ;pvA
        mov     ebx,[esp+16+12]         ;pvB
        mov     ebp,[esp+16+4]          ;pvD

ASSERT_PTR(eax) ;pvA
ASSERT_PTR(ebx) ;pvB
ASSERT_PTR(ebp) ;pvD

        mov     ecx,[eax+0]             ; Read lower 64bits
        mov     esi,[eax+4]
        mov     edx,[ebx+0]                             
        mov     edi,[ebx+4]
        and     ecx,edx
        and     esi,edi
        mov     [ebp+0],ecx             ; Write lower 64bits
        mov     [ebp+4],esi

        mov     ecx,[eax+8]             ; Read upper 64bits
        mov     esi,[eax+12]
        mov     edx,[ebx+8]                             
        mov     edi,[ebx+12]
        and     ecx,edx
        and     esi,edi
        mov     [ebp+8],ecx             ; Write upper 64bits
        mov     [ebp+12],esi

        pop     edi
        pop     esi
        pop     ebx
        pop     ebp
        ret
vmp_pandAsm     endp



;_________________________________________________________________________________________________
;       *** MMX version *** 128bit (un)aligned

        align   16
        .MMX    
        public  vmp_pandAsmMMX
vmp_pandAsmMMX       proc    near pvD:ptr, pvA:ptr, pvB:ptr
;;;        emms
ASSERT_PTR(pvA)
ASSERT_PTR(pvB)
ASSERT_PTR(pvD)

        mov     edx,pvB
        mov     eax,pvA
        mov     ecx,pvD

        movq    mm0,[edx+0]                     ; Read B Data
        movq    mm1,[edx+8]                             
        movq    mm2,[eax+0]                     ; Read A Data
        movq    mm3,[eax+8]

        pand    mm0,mm2                         ; AND lower 64bits
        pand    mm1,mm3                         ; AND upper 64bits

        movq    [ecx+0],mm0                     ; Write Data
        movq    [ecx+8],mm1

;;;        emms
        ret
vmp_pandAsmMMX  endp



;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit non aligned

        align   16
        .XMM
        public  vmp_pandAsmSSE20
vmp_pandAsmSSE20       proc    near pvD:ptr, pvA:ptr, pvB:ptr

ASSERT_PTR(pvA)
ASSERT_PTR(pvB)
ASSERT_PTR(pvD)

        mov     edx,pvB
        mov     eax,pvA

        movdqu  xmm0,[edx]                      ; Read B Data
        movdqu  xmm1,[eax]                      ; Read A Data

        mov     eax,pvD
        pand    xmm0,xmm1                       ; AND 128bits

        movdqu  [eax],xmm0                      ; Write D Data

        ret
vmp_pandAsmSSE20        endp


;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit aligned

        align   16
        .XMM
        public  vmp_pandAsmSSE2
vmp_pandAsmSSE2       proc    near pvD:ptr, pvA:ptr, pvB:ptr

ASSERT_PTR16(pvA)
ASSERT_PTR16(pvB)
ASSERT_PTR16(pvD)

        mov     edx,pvB
        mov     eax,pvA

        movdqa  xmm0,[edx]                      ; Read B Data
        movdqa  xmm1,[eax]                      ; Read A Data

        mov     eax,pvD
        pand    xmm0,xmm1                       ; AND 128bits

        movdqa  [eax],xmm0                      ; Write D Data

        ret
vmp_pandAsmSSE2 endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Logical OR  D=(A | B)
;
;void vmp_por( void * const pvD, const void * const pvA, const void * const pvB )
;{
;       *(((uint32*)pvD)+0) = *(((uint32*)pvA)+0) | *(((uint32*)pvB)+0);
;       *(((uint32*)pvD)+1) = *(((uint32*)pvA)+1) | *(((uint32*)pvB)+1);
;       *(((uint32*)pvD)+2) = *(((uint32*)pvA)+2) | *(((uint32*)pvB)+2);
;       *(((uint32*)pvD)+3) = *(((uint32*)pvA)+3) | *(((uint32*)pvB)+3);
;}
;       *** X86 version *** 128bit (un)aligned

        align   16
        .586
        public  vmp_porAsm
vmp_porAsm       proc    near   ; pvD:ptr, pvA:ptr, pvB:ptr
        push    ebp
        push    ebx
        push    esi
        push    edi

        mov     eax,[esp+16+8]          ;pvA
        mov     ebx,[esp+16+12]         ;pvB
        mov     ebp,[esp+16+4]          ;pvD

ASSERT_PTR(eax) ;pvA
ASSERT_PTR(ebx) ;pvB
ASSERT_PTR(ebp) ;pvD

        mov     ecx,[eax+0]             ; Read lower 64bits
        mov     esi,[eax+4]
        mov     edx,[ebx+0]                             
        mov     edi,[ebx+4]
        or      ecx,edx
        or      esi,edi
        mov     [ebp+0],ecx             ; Write lower 64bits
        mov     [ebp+4],esi

        mov     ecx,[eax+8]             ; Read upper 64bits
        mov     esi,[eax+12]
        mov     edx,[ebx+8]                             
        mov     edi,[ebx+12]
        or      ecx,edx
        or      esi,edi
        mov     [ebp+8],ecx             ; Write upper 64bits
        mov     [ebp+12],esi

        pop     edi
        pop     esi
        pop     ebx
        pop     ebp
        ret
vmp_porAsm      endp


;_________________________________________________________________________________________________
;       *** MMX version *** 128bit (un)aligned

        align   16
        .MMX    
        public  vmp_porAsmMMX
vmp_porAsmMMX       proc    near pvD:ptr, pvA:ptr, pvB:ptr
;;;        emms
ASSERT_PTR(pvA)
ASSERT_PTR(pvB)
ASSERT_PTR(pvD)

        mov     edx,pvB
        mov     eax,pvA
        mov     ecx,pvD

        movq    mm0,[edx+0]                     ; Read B Data
        movq    mm1,[edx+8]                             
        movq    mm2,[eax+0]                     ; Read A Data
        movq    mm3,[eax+8]

        por     mm0,mm2                         ; OR lower 64bits
        por     mm1,mm3                         ; OR upper 64bits

        movq    [ecx+0],mm0                     ; Write D Data
        movq    [ecx+8],mm1

;;;        emms
        ret
vmp_porAsmMMX   endp



;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit non aligned

        align   16
        .XMM
        public  vmp_porAsmSSE20
vmp_porAsmSSE20       proc    near pvD:ptr, pvA:ptr, pvB:ptr

ASSERT_PTR(pvA)
ASSERT_PTR(pvB)
ASSERT_PTR(pvD)

        mov     edx,pvB
        mov     eax,pvA

        movdqu  xmm0,[edx]                      ; Read B Data
        movdqu  xmm1,[eax]                      ; Read A Data

        mov     eax,pvD
        por     xmm0,xmm1                       ; OR 128bits

        movdqu  [eax],xmm0                      ; Write D Data

        ret
vmp_porAsmSSE20 endp


;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit aligned

        align   16
        .XMM
        public  vmp_porAsmSSE2
vmp_porAsmSSE2       proc    near pvD:ptr, pvA:ptr, pvB:ptr

ASSERT_PTR16(pvA)
ASSERT_PTR16(pvB)
ASSERT_PTR16(pvD)

        mov     edx,pvB
        mov     eax,pvA

        movdqa  xmm0,[edx]                      ; Read B Data
        movdqa  xmm1,[eax]                      ; Read A Data

        mov     eax,pvD
        por     xmm0,xmm1                       ; OR 128bits

        movdqa  [eax],xmm0                      ; Write D Data

        ret
vmp_porAsmSSE2  endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Logical XOR (Exclusive OR)  D=(A^B)
;
;void vmp_pxor( void * const pvD, const void * const pvA, const void * const pvB )
;{
;       *(((uint32*)pvD)+0) = *(((uint32*)pvA)+0) ^ *(((uint32*)pvB)+0);
;       *(((uint32*)pvD)+1) = *(((uint32*)pvA)+1) ^ *(((uint32*)pvB)+1);
;       *(((uint32*)pvD)+2) = *(((uint32*)pvA)+2) ^ *(((uint32*)pvB)+2);
;       *(((uint32*)pvD)+3) = *(((uint32*)pvA)+3) ^ *(((uint32*)pvB)+3);
;}
;       *** X86 version *** 128bit (un)aligned

        align   16
        .586
        public  vmp_pxorAsm
vmp_pxorAsm       proc    near  ; pvD:ptr, pvA:ptr, pvB:ptr
        push    ebp
        push    ebx
        push    esi
        push    edi

        mov     eax,[esp+16+8]          ;pvA
        mov     ebx,[esp+16+12]         ;pvB
        mov     ebp,[esp+16+4]          ;pvD

ASSERT_PTR(eax) ;pvA
ASSERT_PTR(ebx) ;pvB
ASSERT_PTR(ebp) ;pvD

        mov     ecx,[eax+0]             ; Read lower 64bits
        mov     esi,[eax+4]
        mov     edx,[ebx+0]                             
        mov     edi,[ebx+4]
        xor     ecx,edx
        xor     esi,edi
        mov     [ebp+0],ecx             ; Write lower 64bits
        mov     [ebp+4],esi

        mov     ecx,[eax+8]             ; Read upper 64bits
        mov     esi,[eax+12]
        mov     edx,[ebx+8]                             
        mov     edi,[ebx+12]
        xor     ecx,edx
        xor     esi,edi
        mov     [ebp+8],ecx             ; Write upper 64bits
        mov     [ebp+12],esi

        pop     edi
        pop     esi
        pop     ebx
        pop     ebp
        ret
vmp_pxorAsm     endp


;_________________________________________________________________________________________________
;       *** MMX version *** 128bit (un)aligned

        align   16
        .MMX    
        public  vmp_pxorAsmMMX
vmp_pxorAsmMMX       proc    near pvD:ptr, pvA:ptr, pvB:ptr
;;;        emms
ASSERT_PTR(pvA)
ASSERT_PTR(pvB)
ASSERT_PTR(pvD)

        mov     edx,pvB
        mov     eax,pvA
        mov     ecx,pvD

        movq    mm0,[edx+0]                     ; Read B Data
        movq    mm1,[edx+8]
        movq    mm2,[eax+0]                     ; Read A Data
        movq    mm3,[eax+8]                             

        pxor    mm0,mm2                         ; XOR lower 64bits
        pxor    mm1,mm3                         ; XOR upper 64bits

        movq    [ecx+0],mm0                     ; Write D Data
        movq    [ecx+8],mm1

;;;        emms
        ret
vmp_pxorAsmMMX  endp



;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit non aligned

        align   16
        .XMM
        public  vmp_pxorAsmSSE20
vmp_pxorAsmSSE20       proc    near pvD:ptr, pvA:ptr, pvB:ptr

ASSERT_PTR(pvA)
ASSERT_PTR(pvB)
ASSERT_PTR(pvD)

        mov     edx,pvB
        mov     eax,pvA

        movdqu  xmm0,[edx]                      ; Read B Data
        movdqu  xmm1,[eax]                      ; Read A Data

        mov     eax,pvD
        pxor    xmm0,xmm1                       ; XOR 128bits

        movdqu  [eax],xmm0                      ; Write D Data

        ret
vmp_pxorAsmSSE20        endp


;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit aligned

        align   16
        .XMM
        public  vmp_pxorAsmSSE2
vmp_pxorAsmSSE2       proc    near pvD:ptr, pvA:ptr, pvB:ptr

ASSERT_PTR16(pvA)
ASSERT_PTR16(pvB)
ASSERT_PTR16(pvD)

        mov     edx,pvB
        mov     eax,pvA

        movdqa  xmm0,[edx]                      ; Read B Data
        movdqa  xmm1,[eax]                      ; Read A Data

        mov     eax,pvD
        pxor    xmm0,xmm1                       ; XOR 128bits

        movdqa  [eax],xmm0                      ; Write D Data

        ret
vmp_pxorAsmSSE2 endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Logical And w/1's Complement  D=(A & !B)
;
;void vmp_pandc( void * const pvD, const void * const pvA, const void * const pvB )
;{
;       *(((uint32*)pvD)+0) = (0xffffffff ^ *(((uint32*)pvB)+0)) & *(((uint32*)pvA)+0);
;       *(((uint32*)pvD)+1) = (0xffffffff ^ *(((uint32*)pvB)+1)) & *(((uint32*)pvA)+1);
;       *(((uint32*)pvD)+2) = (0xffffffff ^ *(((uint32*)pvB)+2)) & *(((uint32*)pvA)+2);
;       *(((uint32*)pvD)+3) = (0xffffffff ^ *(((uint32*)pvB)+3)) & *(((uint32*)pvA)+3);
;}
;       *** X86 version *** 128bit (un)aligned

        align   16
        .586
        public  vmp_pandcAsm
vmp_pandcAsm       proc    near ; pvD:ptr, pvA:ptr, pvB:ptr
        push    ebp
        push    ebx
        push    esi
        push    edi

        mov     eax,[esp+16+8]          ;pvA
        mov     ebx,[esp+16+12]         ;pvB
        mov     ebp,[esp+16+4]          ;pvD

ASSERT_PTR(eax) ;pvA
ASSERT_PTR(ebx) ;pvB
ASSERT_PTR(ebp) ;pvD

        mov     ecx,[eax+0]             ; Read lower 64bits
        mov     esi,[eax+4]
        mov     edx,[ebx+0]                             
        mov     edi,[ebx+4]
        not     edx                     ; Flip B
        not     edi
        and     ecx,edx
        and     esi,edi
        mov     [ebp+0],ecx             ; Write lower 64bits
        mov     [ebp+4],esi

        mov     ecx,[eax+8]             ; Read upper 64bits
        mov     esi,[eax+12]
        mov     edx,[ebx+8]                             
        mov     edi,[ebx+12]
        not     edx                     ; Flip B
        not     edi
        and     ecx,edx
        and     esi,edi
        mov     [ebp+8],ecx             ; Write upper 64bits
        mov     [ebp+12],esi

        pop     edi
        pop     esi
        pop     ebx
        pop     ebp
        ret
vmp_pandcAsm    endp


;_________________________________________________________________________________________________
;       *** MMX version *** 128bit (un)aligned

        align   16
        .MMX    
        public  vmp_pandcAsmMMX
vmp_pandcAsmMMX       proc    near pvD:ptr, pvA:ptr, pvB:ptr
;;;        emms
ASSERT_PTR(pvA)
ASSERT_PTR(pvB)
ASSERT_PTR(pvD)

        mov     edx,pvB
        mov     eax,pvA
        mov     ecx,pvD

        movq    mm0,[edx+0]                     ; Read B Data
        movq    mm1,[edx+8]
        movq    mm2,[eax+0]                     ; Read A Data
        movq    mm3,[eax+8]                             

        pandn   mm0,mm2                         ; ANDc lower 64bits
        pandn   mm1,mm3                         ; ANDc upper 64bits

        movq    [ecx+0],mm0                     ; Write D Data
        movq    [ecx+8],mm1

;;;        emms
        ret
vmp_pandcAsmMMX endp



;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit non aligned

        align   16
        .XMM
        public  vmp_pandcAsmSSE20
vmp_pandcAsmSSE20       proc    near pvD:ptr, pvA:ptr, pvB:ptr

ASSERT_PTR(pvB)
ASSERT_PTR(pvA)
ASSERT_PTR(pvD)

        mov     edx,pvB
        mov     eax,pvA

        movdqu  xmm0,[edx]                      ; Read B Data
        movdqu  xmm1,[eax]                      ; Read A Data

        mov     eax,pvD
        pandn   xmm0,xmm1                       ; = (B' AND A) 128bits

        movdqu  [eax],xmm0                      ; Write D Data

        ret
vmp_pandcAsmSSE20       endp


;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit aligned

        align   16
        .XMM
        public  vmp_pandcAsmSSE2
vmp_pandcAsmSSE2       proc    near pvD:ptr, pvA:ptr, pvB:ptr

ASSERT_PTR16(pvB)
ASSERT_PTR16(pvA)
ASSERT_PTR16(pvD)

        mov     edx,pvB
        mov     eax,pvA

        movdqa  xmm0,[edx]                      ; Read B Data
        movdqa  xmm1,[eax]                      ; Read A Data

        mov     eax,pvD
        pandn   xmm0,xmm1                       ; = (B' AND A) 128bits

        movdqa  [eax],xmm0                      ; Write D Data

        ret
vmp_pandcAsmSSE2        endp



;_________________________________________________________________________________________________
;=================================================================================================
;       Logical NOR  1's complement of OR  D=!(A | B)
;
;void vmp_pnor( void * const pvD, const void * const pvA, const void * const pvB )
;{
;       *(((uint32*)pvD)+0) = 0xffffffff ^ ( *(((uint32*)pvA)+0) | *(((uint32*)pvB)+0) );
;       *(((uint32*)pvD)+1) = 0xffffffff ^ ( *(((uint32*)pvA)+1) | *(((uint32*)pvB)+1) );
;       *(((uint32*)pvD)+2) = 0xffffffff ^ ( *(((uint32*)pvA)+2) | *(((uint32*)pvB)+2) );
;       *(((uint32*)pvD)+3) = 0xffffffff ^ ( *(((uint32*)pvA)+3) | *(((uint32*)pvB)+3) );
;}
;       *** X86 version *** 128bit (un)aligned

        align   16
        .586
        public  vmp_pnorAsm
vmp_pnorAsm       proc    near  ; pvD:ptr, pvA:ptr, pvB:ptr
        push    ebp
        push    ebx
        push    esi
        push    edi

        mov     eax,[esp+16+8]          ;pvA
        mov     ebx,[esp+16+12]         ;pvB
        mov     ebp,[esp+16+4]          ;pvD

ASSERT_PTR(eax) ;pvA
ASSERT_PTR(ebx) ;pvB
ASSERT_PTR(ebp) ;pvD

        mov     ecx,[eax+0]             ; Read lower 64bits
        mov     esi,[eax+4]
        mov     edx,[ebx+0]                             
        mov     edi,[ebx+4]
        or      ecx,edx
        or      esi,edi
        not     ecx                     ; Flip
        not     esi
        mov     [ebp+0],ecx             ; Write lower 64bits
        mov     [ebp+4],esi

        mov     ecx,[eax+8]             ; Read upper 64bits
        mov     esi,[eax+12]
        mov     edx,[ebx+8]                             
        mov     edi,[ebx+12]
        or      ecx,edx
        or      esi,edi
        not     ecx                     ; Flip
        not     esi
        mov     [ebp+8],ecx             ; Write upper 64bits
        mov     [ebp+12],esi

        pop     edi
        pop     esi
        pop     ebx
        pop     ebp
        ret
vmp_pnorAsm     endp


;_________________________________________________________________________________________________
;       *** MMX version *** 128bit (un)aligned

        align   16
        .MMX    
        public  vmp_pnorAsmMMX
vmp_pnorAsmMMX       proc    near pvD:ptr, pvA:ptr, pvB:ptr
;;;        emms
ASSERT_PTR(pvB)
ASSERT_PTR(pvA)
ASSERT_PTR(pvD)

        mov     edx,pvB
        mov     eax,pvA
        mov     ecx,pvD

        movq    mm0,[edx+0]                     ; Read B Data
        movq    mm1,[edx+8]                             
        movq    mm2,[eax+0]                     ; Read A Data
        movq    mm3,[eax+8]

        por     mm0,mm2                         ; OR lower 64bits
        por     mm1,mm3                         ; OR upper 64bits

        pxor    mm0,NotMaskQ                    ; Flip Bits
        pxor    mm1,NotMaskQ

        movq    [ecx+0],mm0                     ; Write Data
        movq    [ecx+8],mm1

;;;        emms
        ret
vmp_pnorAsmMMX  endp



;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit non aligned

        align   16
        .XMM
        public  vmp_pnorAsmSSE20
vmp_pnorAsmSSE20       proc    near pvD:ptr, pvA:ptr, pvB:ptr

ASSERT_PTR(pvB)
ASSERT_PTR(pvA)
ASSERT_PTR(pvD)

        mov     edx,pvB
        mov     eax,pvA

        movdqu  xmm0,[edx]                      ; Read B Data
        movdqu  xmm1,[eax]                      ; Read A Data

        mov     eax,pvD
        por     xmm0,xmm1                       ; OR 128bits
        pxor    xmm0,NotMaskO                   ; Flip Bits

        movdqu  [eax],xmm0                      ; Write D Data

        ret
vmp_pnorAsmSSE20        endp


;_________________________________________________________________________________________________
;       *** SSE2 version *** 128bit aligned

        align   16
        .XMM
        public  vmp_pnorAsmSSE2
vmp_pnorAsmSSE2       proc    near pvD:ptr, pvA:ptr, pvB:ptr

ASSERT_PTR16(pvA)
ASSERT_PTR16(pvB)
ASSERT_PTR16(pvD)

        mov     edx,pvB
        mov     eax,pvA

        movdqa  xmm0,[edx]                      ; Read B Data
        movdqa  xmm1,[eax]                      ; Read A Data

        mov     eax,pvD
        por     xmm0,xmm1                       ; OR 128bits
        pxor    xmm0,NotMaskO                   ; Flip Bits

        movdqa  [eax],xmm0                      ; Write D Data

        ret
vmp_pnorAsmSSE2 endp



pboolX86  ends
        end




